@@ -277,6 +277,15 @@ namespace Konversation
     {
         Konversation::ServerGroupSettingsPtr serverGroup = Preferences::serverGroupById(item->data(0,ServerGroupId).toInt());
         serverGroup->setExpanded(false);
+
+        for(int i = 0; i < item->childCount(); i++)
+        {
+          if(item->child(i)->isSelected())
+          {
+            item->child(i)->setSelected(false);
+            item->setSelected(true);
+          }
+        }
     }
 
     void ServerListDialog::slotAboutToMove()
@@ -436,6 +445,7 @@ namespace Konversation
             serverItem->setData(0,SortIndex,i);
             serverItem->setData(0,IsServer,true);
             serverItem->setData(0,ServerId,i);
+            serverItem->setFirstColumnSpanned(true);
             // Initialize a pointer to the new location of the last edited server
             if (m_selectedItem && m_selectedServer==(*serverIt))
                 m_selectedItemPtr = serverItem;
