@@ -14,13 +14,15 @@
 
 #include <QObject>
 
-
 class ChatWindow;
 class Application;
 class MainWindow;
 
 namespace Konversation
 {
+    namespace DCC {
+        class Transfer;
+    }
 
     class NotificationHandler : public QObject
     {
@@ -39,8 +41,8 @@ namespace Konversation
             void nickChange(ChatWindow* chatWin, const QString& oldNick, const QString& newNick);
             void dccIncoming(ChatWindow* chatWin, const QString& fromNick);
             void dccError(ChatWindow* chatWin, const QString& error);
-            void dccTransferDone(ChatWindow* chatWin, const QString& file);
-            void mode(ChatWindow* chatWin, const QString& nick);
+            void dccTransferDone(ChatWindow* chatWin, const QString& file, Konversation::DCC::Transfer* transfer);
+            void mode(ChatWindow* chatWin, const QString& nick, const QString& subject, const QString& change);
             void query(ChatWindow* chatWin, const QString& fromNick);
             void queryMessage(ChatWindow* chatWin, const QString& fromNick, const QString& message);
             void nickOnline(ChatWindow* chatWin, const QString& nick);
